<?php

namespace Drupal\leaflet_more_maps\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form to enter global settings and to assemble custom maps from overlays.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'leaflet_more_maps_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('leaflet_more_maps.settings');

    $form['global_settings'] = [
      '#type' => 'details',
      '#title' => t('Global settings'),
      '#open' => TRUE,
    ];
    $form['global_settings']['thunderforest_api_key'] = [
      '#type' => 'textfield',
      '#title' => t('OSM Thunderforest API Key'),
      '#default_value' => $config->get('thunderforest_api_key', ''),
      '#description' => t('If you use <a href="@thunderforest" target="_thunderforest_maps">Thunderforest</a> maps, please obtain an API key <a href="@api_keys" target="_thunderforest_api_key">here</a> and paste it above.',
        [
          '@thunderforest' => 'https://www.thunderforest.com/maps',
          '@api_keys' => 'http://www.thunderforest.com/docs/apikeys/',
        ]),
    ];

    $map_info = [];

    _leaflet_more_maps_assemble_default_map_info($map_info);

    $all_layer_keys = [];
    foreach ($map_info as $map_key => $map) {
      foreach ($map['layers'] as $layer_key => $layer) {
        // Unique.
        $all_layer_keys["$map_key $layer_key"] = "$map_key $layer_key";
      }
    }
    $custom_map_layers = $config->get('leaflet_more_maps_custom_maps', []);

    if (empty($custom_map_layers)) {
      for ($i = 1; $i <= LEAFLET_MORE_MAPS_NO_CUSTOM_MAPS; $i++) {
        $custom_map_layers[$i] = [
          'map-key' => '',
          'layer-keys' => [],
          'reverse-order' => FALSE,
        ];
      }
    }
    for ($i = 1; $i <= LEAFLET_MORE_MAPS_NO_CUSTOM_MAPS; $i++) {
      $form['map'][$i] = [
        '#type' => 'details',
        '#open' => $i <= 1,
        '#title' => $this->t('Custom map #@number layer selection', ['@number' => $i]),
      ];
      $form['map'][$i]['map-key'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name of custom map #@number in the administrative UI', ['@number' => $i]),
        '#default_value' => $custom_map_layers[$i]['map-key'],
        '#description' => $this->t('Use a blank field to remove this layer configuration from the set of selectable maps.'),
      ];
      $form['map'][$i]['layer-keys'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Select one or more layers to be included in this map.'),
        '#options' => $all_layer_keys,
        '#default_value' => $custom_map_layers[$i]['layer-keys'],
        '#description' => $this->t('If you select two or more layers, these will be selectable via radio buttons in the layer switcher on your map.'),
      ];
      $form['map'][$i]['reverse-order'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Reverse order in layer switcher'),
        '#default_value' => $custom_map_layers[$i]['reverse-order'],
        '#description' => $this->t('The last layer in the switcher will be the default.'),
      ];
      $form['map'][$i]['map-key']['#parents'] = ['map', $i, 'map-key'];
      $form['map'][$i]['layer-keys']['#parents'] = ['map', $i, 'layer-keys'];
      $form['map'][$i]['reverse-order']['#parents'] =
        ['map', $i, 'reverse-order'];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $custom_maps = $form_state->getValue('map');

    // Clear out the unticked boxes before saving the form.
    foreach ($custom_maps as $i => $custom_map) {
      $custom_map['layer-keys'] = array_filter($custom_map['layer-keys']);
      if (empty($custom_map['map_key']) || empty($custom_map['layer_keys'])) {
        unset($custom_maps[$i]);
      }
    }

    $this->config('leaflet_more_maps.settings')
      ->set('thunderforest_api_key', $form_state->getValue('thunderforest_api_key'))
      ->set('leaflet_more_maps_custom_maps', $custom_maps)
      ->save();

    parent::submitForm($form, $form_state);
    // @todo Need to refresh config cache.
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['leaflet_more_maps.settings'];
  }

}
